@@ -1,19 +1,6 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
+// SPDX-License-Identifier: GPL-2.0+
 /*
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
- *
- * Copyright 2013 Red Hat, Inc.
+ * Copyright (C) 2013 Red Hat, Inc.
  */
 
 /**
@@ -240,7 +227,7 @@ activate_connection (NMConnection *connection,
                      NMObject     *specific_object)
 {
 	NmtNewtForm *form;
-	gs_unref_object NMSecretAgentOld *agent = NULL;
+	gs_unref_object NMSecretAgentSimple *agent = NULL;
 	NmtNewtWidget *label;
 	NmtSyncOp op;
 	const char *specific_object_path;
@@ -257,7 +244,7 @@ activate_connection (NMConnection *connection,
 	agent = nm_secret_agent_simple_new ("nmtui");
 	if (agent) {
 		if (connection) {
-			nm_secret_agent_simple_enable (NM_SECRET_AGENT_SIMPLE (agent),
+			nm_secret_agent_simple_enable (agent,
 			                               nm_object_get_path (NM_OBJECT (connection)));
 		}
 		g_signal_connect (agent,
@@ -303,7 +290,7 @@ activate_connection (NMConnection *connection,
 	if (agent && !connection) {
 		connection = NM_CONNECTION (nm_active_connection_get_connection (ac));
 		if (connection) {
-			nm_secret_agent_simple_enable (NM_SECRET_AGENT_SIMPLE (agent),
+			nm_secret_agent_simple_enable (agent,
 			                               nm_object_get_path (NM_OBJECT (connection)));
 		}
 	}
@@ -341,7 +328,7 @@ activate_connection (NMConnection *connection,
 	g_object_unref (form);
 
 	if (agent)
-		nm_secret_agent_old_unregister (agent, NULL, NULL);
+		nm_secret_agent_old_unregister (NM_SECRET_AGENT_OLD (agent), NULL, NULL);
 }
 
 static void
