@@ -1,18 +1,5 @@
 #!/usr/bin/env python
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along
-# with this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+# SPDX-License-Identifier: GPL-2.0+
 #
 # Copyright (C) 2010 - 2012 Red Hat, Inc.
 #
@@ -38,7 +25,7 @@ s_con = dbus.Dictionary({
 
 addr1 = dbus.Dictionary({
     'address': '10.1.2.3',
-    'prefix': dbus.UInt32(8L)})
+    'prefix': dbus.UInt32(8)})
 s_ip4 = dbus.Dictionary({
             'address-data': dbus.Array([addr1], signature=dbus.Signature('a{sv}')),
             'gateway': '10.1.2.1',
@@ -53,7 +40,7 @@ con = dbus.Dictionary({
     'ipv6': s_ip6})
 
 
-print "Creating connection:", s_con['id'], "-", s_con['uuid']
+print("Creating connection:", s_con['id'], "-", s_con['uuid'])
 
 bus = dbus.SystemBus()
 proxy = bus.get_object("org.freedesktop.NetworkManager", "/org/freedesktop/NetworkManager/Settings")
