@@ -1,18 +1,5 @@
 #!/usr/bin/env python
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along
-# with this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+# SPDX-License-Identifier: GPL-2.0+
 #
 # Copyright (C) 2011 Red Hat, Inc.
 #
@@ -20,7 +7,7 @@
 import dbus, uuid
 
 def path_to_value(path):
-    return dbus.ByteArray("file://" + path + "\0")
+    return dbus.ByteArray("file://".encode("utf-8") + path.encode("utf-8") + "\0".encode("utf-8"))
 
 s_con = dbus.Dictionary({
     'type': '802-11-wireless',
@@ -28,7 +15,7 @@ s_con = dbus.Dictionary({
     'id': 'My Wifi'})
 
 s_wifi = dbus.Dictionary({
-    'ssid': dbus.ByteArray("homewifi"),
+    'ssid': dbus.ByteArray("homewifi".encode("utf-8")),
     'security': '802-11-wireless-security'})
 
 s_wsec = dbus.Dictionary({'key-mgmt': 'wpa-eap'})
