@@ -1,21 +1,6 @@
 #!/usr/bin/env python
 # coding=utf-8
-# -*- Mode: Python; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
-# vim: ft=python ts=4 sts=4 sw=4 et ai
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along
-# with this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+# SPDX-License-Identifier: GPL-2.0+
 #
 # Copyright (C) 2013 Red Hat, Inc.
 #
@@ -49,8 +34,8 @@ def print_device_info(device):
     if active_ap is not None:
         ssid = ssid_to_utf8(active_ap)
     info = "Device: %s | Driver: %s | Active AP: %s" % (dev.get_iface(), dev.get_driver(), ssid)
-    print info
-    print '=' * len(info)
+    print(info)
+    print('=' * len(info))
 
 def mode_to_string(mode):
     if mode == getattr(NM, '80211Mode').INFRA:
@@ -114,16 +99,16 @@ def print_ap_info(ap):
     flags = ap.get_flags()
     wpa_flags = ap.get_wpa_flags()
     rsn_flags = ap.get_rsn_flags()
-    print "SSID:      %s"      % (ssid_to_utf8(ap))
-    print "BSSID:     %s"      % (ap.get_bssid())
-    print "Frequency: %s"      % (frequency)
-    print "Channel:   %s"      % (NM.utils_wifi_freq_to_channel(frequency))
-    print "Mode:      %s"      % (mode_to_string(ap.get_mode()))
-    print "Flags:     %s"      % (flags_to_string(flags))
-    print "WPA flags: %s"      % (security_flags_to_string(wpa_flags))
-    print "RSN flags: %s"      % (security_flags_to_string(rsn_flags))
-    print "Security:  %s"      % (flags_to_security(flags, wpa_flags, rsn_flags))
-    print "Strength:  %s %s%%" % (NM.utils_wifi_strength_bars(strength), strength)
+    print("SSID:      %s"      % (ssid_to_utf8(ap)))
+    print("BSSID:     %s"      % (ap.get_bssid()))
+    print("Frequency: %s"      % (frequency))
+    print("Channel:   %s"      % (NM.utils_wifi_freq_to_channel(frequency)))
+    print("Mode:      %s"      % (mode_to_string(ap.get_mode())))
+    print("Flags:     %s"      % (flags_to_string(flags)))
+    print("WPA flags: %s"      % (security_flags_to_string(wpa_flags)))
+    print("RSN flags: %s"      % (security_flags_to_string(rsn_flags)))
+    print("Security:  %s"      % (flags_to_security(flags, wpa_flags, rsn_flags)))
+    print("Strength:  %s %s%%" % (NM.utils_wifi_strength_bars(strength), strength))
     print
 
 if __name__ == "__main__":
