@@ -1,23 +1,7 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-
+// SPDX-License-Identifier: LGPL-2.1+
 /*
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA.
- *
- * Copyright 2007 - 2011 Red Hat, Inc.
- * Copyright 2007 - 2008 Novell, Inc.
+ * Copyright (C) 2007 - 2011 Red Hat, Inc.
+ * Copyright (C) 2007 - 2008 Novell, Inc.
  */
 
 #ifndef __NM_SETTING_CDMA_H__
@@ -27,7 +11,7 @@
 #error "Only <NetworkManager.h> can be included directly."
 #endif
 
-#include <nm-setting.h>
+#include "nm-setting.h"
 
 G_BEGIN_DECLS
 
@@ -44,9 +28,12 @@ G_BEGIN_DECLS
 #define NM_SETTING_CDMA_USERNAME       "username"
 #define NM_SETTING_CDMA_PASSWORD       "password"
 #define NM_SETTING_CDMA_PASSWORD_FLAGS "password-flags"
+#define NM_SETTING_CDMA_MTU            "mtu"
 
 /**
  * NMSettingCdma:
+ *
+ * CDMA-based Mobile Broadband Settings
  */
 struct _NMSettingCdma {
 	NMSetting parent;
@@ -67,6 +54,9 @@ const char *nm_setting_cdma_get_username (NMSettingCdma *setting);
 const char *nm_setting_cdma_get_password (NMSettingCdma *setting);
 NMSettingSecretFlags nm_setting_cdma_get_password_flags (NMSettingCdma *setting);
 
+NM_AVAILABLE_IN_1_8
+guint32 nm_setting_cdma_get_mtu          (NMSettingCdma *setting);
+
 G_END_DECLS
 
 #endif /* __NM_SETTING_CDMA_H__ */
