@@ -1,20 +1,6 @@
+// SPDX-License-Identifier: LGPL-2.1+
 /*
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA.
- *
- * Copyright 2015 Red Hat, Inc.
+ * Copyright (C) 2015 Red Hat, Inc.
  */
 
 #include "nm-default.h"
@@ -26,33 +12,9 @@
 #include "nm-utils.h"
 #include "nm-object-private.h"
 
-G_DEFINE_TYPE (NMDeviceVxlan, nm_device_vxlan, NM_TYPE_DEVICE)
+/*****************************************************************************/
 
-#define NM_DEVICE_VXLAN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_DEVICE_VXLAN, NMDeviceVxlanPrivate))
-
-typedef struct {
-	NMDevice *parent;
-	char *hw_address;
-	gboolean carrier;
-	guint id;
-	char *group;
-	char *local;
-	guint src_port_min;
-	guint src_port_max;
-	guint dst_port;
-	guint tos;
-	guint ttl;
-	guint limit;
-	gboolean learning;
-	guint ageing;
-	gboolean proxy;
-	gboolean rsc;
-	gboolean l2miss;
-	gboolean l3miss;
-} NMDeviceVxlanPrivate;
-
-enum {
-	PROP_0,
+NM_GOBJECT_PROPERTIES_DEFINE_BASE (
 	PROP_HW_ADDRESS,
 	PROP_CARRIER,
 	PROP_PARENT,
@@ -71,10 +33,43 @@ enum {
 	PROP_RSC,
 	PROP_L2MISS,
 	PROP_L3MISS,
+);
 
-	LAST_PROP
+typedef struct {
+	NMLDBusPropertyO parent;
+	char *hw_address;
+	char *group;
+	char *local;
+	guint32 id;
+	guint32 limit;
+	guint32 ageing;
+	guint16 src_port_min;
+	guint16 src_port_max;
+	guint16 dst_port;
+	guint8 tos;
+	guint8 ttl;
+	bool learning;
+	bool proxy;
+	bool rsc;
+	bool l2miss;
+	bool l3miss;
+} NMDeviceVxlanPrivate;
+
+struct _NMDeviceVxlan {
+	NMDevice parent;
+	NMDeviceVxlanPrivate _priv;
 };
 
+struct _NMDeviceVxlanClass {
+	NMDeviceClass parent;
+};
+
+G_DEFINE_TYPE (NMDeviceVxlan, nm_device_vxlan, NM_TYPE_DEVICE)
+
+#define NM_DEVICE_VXLAN_GET_PRIVATE(self) _NM_GET_PRIVATE(self, NMDeviceVxlan, NM_IS_DEVICE_VXLAN, NMObject, NMDevice)
+
+/*****************************************************************************/
+
 /**
  * nm_device_vxlan_get_hw_address:
  * @device: a #NMDeviceVxlan
@@ -91,7 +86,7 @@ nm_device_vxlan_get_hw_address (NMDeviceVxlan *device)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_VXLAN (device), NULL);
 
-	return nm_str_not_empty (NM_DEVICE_VXLAN_GET_PRIVATE (device)->hw_address);
+	return _nml_coerce_property_str_not_empty (NM_DEVICE_VXLAN_GET_PRIVATE (device)->hw_address);
 }
 
 /**
@@ -103,13 +98,16 @@ nm_device_vxlan_get_hw_address (NMDeviceVxlan *device)
  * Returns: %TRUE if the device has carrier.
  *
  * Since: 1.2
+ *
+ * This property is not implemented yet, and the function always returns
+ * FALSE.
  **/
 gboolean
 nm_device_vxlan_get_carrier (NMDeviceVxlan *device)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_VXLAN (device), FALSE);
 
-	return NM_DEVICE_VXLAN_GET_PRIVATE (device)->carrier;
+	return FALSE;
 }
 
 /**
@@ -125,7 +123,7 @@ nm_device_vxlan_get_parent (NMDeviceVxlan *device)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_VXLAN (device), NULL);
 
-	return NM_DEVICE_VXLAN_GET_PRIVATE (device)->parent;
+	return nml_dbus_property_o_get_obj (&NM_DEVICE_VXLAN_GET_PRIVATE (device)->parent);
 }
 
 /**
@@ -158,7 +156,7 @@ nm_device_vxlan_get_group (NMDeviceVxlan *device)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_VXLAN (device), NULL);
 
-	return nm_str_not_empty (NM_DEVICE_VXLAN_GET_PRIVATE (device)->group);
+	return _nml_coerce_property_str_not_empty (NM_DEVICE_VXLAN_GET_PRIVATE (device)->group);
 }
 
 /**
@@ -174,7 +172,7 @@ nm_device_vxlan_get_local (NMDeviceVxlan *device)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_VXLAN (device), NULL);
 
-	return nm_str_not_empty (NM_DEVICE_VXLAN_GET_PRIVATE (device)->local);
+	return _nml_coerce_property_str_not_empty (NM_DEVICE_VXLAN_GET_PRIVATE (device)->local);
 }
 
 /**
@@ -413,46 +411,12 @@ nm_device_vxlan_init (NMDeviceVxlan *device)
 {
 }
 
-static void
-init_dbus (NMObject *object)
-{
-	NMDeviceVxlanPrivate *priv = NM_DEVICE_VXLAN_GET_PRIVATE (object);
-	const NMPropertiesInfo property_info[] = {
-		{ NM_DEVICE_VXLAN_HW_ADDRESS,   &priv->hw_address },
-		{ NM_DEVICE_VXLAN_CARRIER,      &priv->carrier },
-		{ NM_DEVICE_VXLAN_PARENT,       &priv->parent, NULL, NM_TYPE_DEVICE },
-		{ NM_DEVICE_VXLAN_ID,           &priv->id },
-		{ NM_DEVICE_VXLAN_GROUP,        &priv->group },
-		{ NM_DEVICE_VXLAN_LOCAL,        &priv->local },
-		{ NM_DEVICE_VXLAN_SRC_PORT_MIN, &priv->src_port_min },
-		{ NM_DEVICE_VXLAN_SRC_PORT_MAX, &priv->src_port_max },
-		{ NM_DEVICE_VXLAN_DST_PORT,     &priv->dst_port },
-		{ NM_DEVICE_VXLAN_TOS,          &priv->tos },
-		{ NM_DEVICE_VXLAN_TTL,          &priv->ttl },
-		{ NM_DEVICE_VXLAN_LIMIT,        &priv->limit },
-		{ NM_DEVICE_VXLAN_LEARNING,     &priv->learning },
-		{ NM_DEVICE_VXLAN_AGEING,       &priv->ageing },
-		{ NM_DEVICE_VXLAN_PROXY,        &priv->proxy },
-		{ NM_DEVICE_VXLAN_RSC,          &priv->rsc },
-		{ NM_DEVICE_VXLAN_L2MISS,       &priv->l2miss },
-		{ NM_DEVICE_VXLAN_L3MISS,       &priv->l3miss },
-		{ NULL },
-	};
-
-	NM_OBJECT_CLASS (nm_device_vxlan_parent_class)->init_dbus (object);
-
-	_nm_object_register_properties (object,
-	                                NM_DBUS_INTERFACE_DEVICE_VXLAN,
-	                                property_info);
-}
-
 static void
 finalize (GObject *object)
 {
 	NMDeviceVxlanPrivate *priv = NM_DEVICE_VXLAN_GET_PRIVATE (object);
 
 	g_free (priv->hw_address);
-	g_clear_object (&priv->parent);
 	g_free (priv->group);
 	g_free (priv->local);
 
@@ -528,26 +492,48 @@ get_property (GObject *object,
 	}
 }
 
+const NMLDBusMetaIface _nml_dbus_meta_iface_nm_device_vxlan = NML_DBUS_META_IFACE_INIT_PROP (
+	NM_DBUS_INTERFACE_DEVICE_VXLAN,
+	nm_device_vxlan_get_type,
+	NML_DBUS_META_INTERFACE_PRIO_INSTANTIATE_HIGH,
+	NML_DBUS_META_IFACE_DBUS_PROPERTIES (
+		NML_DBUS_META_PROPERTY_INIT_U      ("Ageing",     PROP_AGEING,       NMDeviceVxlan, _priv.ageing                           ),
+		NML_DBUS_META_PROPERTY_INIT_Q      ("DstPort",    PROP_DST_PORT,     NMDeviceVxlan, _priv.dst_port                         ),
+		NML_DBUS_META_PROPERTY_INIT_S      ("Group",      PROP_GROUP,        NMDeviceVxlan, _priv.group                            ),
+		NML_DBUS_META_PROPERTY_INIT_S      ("HwAddress",  PROP_HW_ADDRESS,   NMDeviceVxlan, _priv.hw_address                       ),
+		NML_DBUS_META_PROPERTY_INIT_U      ("Id",         PROP_ID,           NMDeviceVxlan, _priv.id                               ),
+		NML_DBUS_META_PROPERTY_INIT_B      ("L2miss",     PROP_L2MISS,       NMDeviceVxlan, _priv.l2miss                           ),
+		NML_DBUS_META_PROPERTY_INIT_B      ("L3miss",     PROP_L3MISS,       NMDeviceVxlan, _priv.l3miss                           ),
+		NML_DBUS_META_PROPERTY_INIT_B      ("Learning",   PROP_LEARNING,     NMDeviceVxlan, _priv.learning                         ),
+		NML_DBUS_META_PROPERTY_INIT_U      ("Limit",      PROP_LIMIT,        NMDeviceVxlan, _priv.limit                            ),
+		NML_DBUS_META_PROPERTY_INIT_S      ("Local",      PROP_LOCAL,        NMDeviceVxlan, _priv.local                            ),
+		NML_DBUS_META_PROPERTY_INIT_O_PROP ("Parent",     PROP_PARENT,       NMDeviceVxlan, _priv.parent,       nm_device_get_type ),
+		NML_DBUS_META_PROPERTY_INIT_B      ("Proxy",      PROP_PROXY,        NMDeviceVxlan, _priv.proxy                            ),
+		NML_DBUS_META_PROPERTY_INIT_B      ("Rsc",        PROP_RSC,          NMDeviceVxlan, _priv.rsc                              ),
+		NML_DBUS_META_PROPERTY_INIT_Q      ("SrcPortMax", PROP_SRC_PORT_MAX, NMDeviceVxlan, _priv.src_port_max                     ),
+		NML_DBUS_META_PROPERTY_INIT_Q      ("SrcPortMin", PROP_SRC_PORT_MIN, NMDeviceVxlan, _priv.src_port_min                     ),
+		NML_DBUS_META_PROPERTY_INIT_Y      ("Tos",        PROP_TOS,          NMDeviceVxlan, _priv.tos                              ),
+		NML_DBUS_META_PROPERTY_INIT_Y      ("Ttl",        PROP_TTL,          NMDeviceVxlan, _priv.ttl                              ),
+	),
+);
+
 static void
-nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
+nm_device_vxlan_class_init (NMDeviceVxlanClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (vxlan_class);
-	NMObjectClass *nm_object_class = NM_OBJECT_CLASS (vxlan_class);
-	NMDeviceClass *device_class = NM_DEVICE_CLASS (vxlan_class);
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	NMObjectClass *nm_object_class = NM_OBJECT_CLASS (klass);
+	NMDeviceClass *device_class = NM_DEVICE_CLASS (klass);
 
-	g_type_class_add_private (vxlan_class, sizeof (NMDeviceVxlanPrivate));
-
-	/* virtual methods */
-	object_class->finalize = finalize;
 	object_class->get_property = get_property;
+	object_class->finalize     = finalize;
 
-	nm_object_class->init_dbus = init_dbus;
+	_NM_OBJECT_CLASS_INIT_PRIV_PTR_DIRECT (nm_object_class, NMDeviceVxlan);
+
+	_NM_OBJECT_CLASS_INIT_PROPERTY_O_FIELDS_1 (nm_object_class, NMDeviceVxlanPrivate, parent);
 
 	device_class->connection_compatible = connection_compatible;
-	device_class->get_setting_type = get_setting_type;
-	device_class->get_hw_address = get_hw_address;
-
-	/* properties */
+	device_class->get_setting_type      = get_setting_type;
+	device_class->get_hw_address        = get_hw_address;
 
 	/**
 	 * NMDeviceVxlan:hw-address:
@@ -556,12 +542,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 **/
-	g_object_class_install_property
-		(object_class, PROP_HW_ADDRESS,
-		 g_param_spec_string (NM_DEVICE_VXLAN_HW_ADDRESS, "", "",
-		                      NULL,
-		                      G_PARAM_READABLE |
-		                      G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_HW_ADDRESS] =
+	    g_param_spec_string (NM_DEVICE_VXLAN_HW_ADDRESS, "", "",
+	                         NULL,
+	                         G_PARAM_READABLE |
+	                         G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:carrier:
@@ -569,13 +554,14 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 * Whether the device has carrier.
 	 *
 	 * Since: 1.2
+	 *
+	 * This property is not implemented yet, and the property is always FALSE.
 	 **/
-	g_object_class_install_property
-		(object_class, PROP_CARRIER,
-		 g_param_spec_boolean (NM_DEVICE_VXLAN_CARRIER, "", "",
-		                       FALSE,
-		                       G_PARAM_READABLE |
-		                       G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_CARRIER] =
+	    g_param_spec_boolean (NM_DEVICE_VXLAN_CARRIER, "", "",
+	                          FALSE,
+	                          G_PARAM_READABLE |
+	                          G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:parent:
@@ -584,12 +570,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 **/
-	g_object_class_install_property
-	    (object_class, PROP_PARENT,
-	     g_param_spec_object (NM_DEVICE_VXLAN_PARENT, "", "",
-	                          NM_TYPE_DEVICE,
-	                          G_PARAM_READABLE |
-	                          G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_PARENT] =
+	    g_param_spec_object (NM_DEVICE_VXLAN_PARENT, "", "",
+	                         NM_TYPE_DEVICE,
+	                         G_PARAM_READABLE |
+	                         G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:id:
@@ -598,12 +583,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 **/
-	g_object_class_install_property
-		(object_class, PROP_ID,
-		 g_param_spec_uint (NM_DEVICE_VXLAN_ID, "", "",
-		                    0, (1 << 24) - 1, 0,
-		                    G_PARAM_READABLE |
-		                    G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_ID] =
+	    g_param_spec_uint (NM_DEVICE_VXLAN_ID, "", "",
+	                       0, (1 << 24) - 1, 0,
+	                       G_PARAM_READABLE |
+	                       G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:group:
@@ -614,12 +598,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_GROUP,
-		 g_param_spec_string (NM_DEVICE_VXLAN_GROUP, "", "",
-		                      NULL,
-		                      G_PARAM_READABLE |
-		                      G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_GROUP] =
+	    g_param_spec_string (NM_DEVICE_VXLAN_GROUP, "", "",
+	                         NULL,
+	                         G_PARAM_READABLE |
+	                         G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:local:
@@ -628,12 +611,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_LOCAL,
-		 g_param_spec_string (NM_DEVICE_VXLAN_LOCAL, "", "",
-		                      NULL,
-		                      G_PARAM_READABLE |
-		                      G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_LOCAL] =
+	    g_param_spec_string (NM_DEVICE_VXLAN_LOCAL, "", "",
+	                         NULL,
+	                         G_PARAM_READABLE |
+	                         G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:tos:
@@ -642,12 +624,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_TOS,
-		 g_param_spec_uchar (NM_DEVICE_VXLAN_TOS, "", "",
-		                     0, 255, 0,
-		                     G_PARAM_READABLE |
-		                     G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_TOS] =
+	    g_param_spec_uchar (NM_DEVICE_VXLAN_TOS, "", "",
+	                        0, 255, 0,
+	                        G_PARAM_READABLE |
+	                        G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:ttl:
@@ -656,12 +637,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_TTL,
-		 g_param_spec_uchar (NM_DEVICE_VXLAN_TTL, "", "",
-		                     0, 255, 0,
-		                     G_PARAM_READABLE |
-		                     G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_TTL] =
+	    g_param_spec_uchar (NM_DEVICE_VXLAN_TTL, "", "",
+	                        0, 255, 0,
+	                        G_PARAM_READABLE |
+	                        G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:learning:
@@ -671,12 +651,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_LEARNING,
-		 g_param_spec_boolean (NM_DEVICE_VXLAN_LEARNING, "", "",
-		                       TRUE,
-		                       G_PARAM_READABLE |
-		                       G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_LEARNING] =
+	    g_param_spec_boolean (NM_DEVICE_VXLAN_LEARNING, "", "",
+	                          TRUE,
+	                          G_PARAM_READABLE |
+	                          G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:ageing:
@@ -685,12 +664,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_AGEING,
-		 g_param_spec_uint (NM_DEVICE_VXLAN_AGEING, "", "",
-		                    0, G_MAXUINT32, 0,
-		                    G_PARAM_READABLE |
-		                    G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_AGEING] =
+	    g_param_spec_uint (NM_DEVICE_VXLAN_AGEING, "", "",
+	                       0, G_MAXUINT32, 0,
+	                       G_PARAM_READABLE |
+	                       G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:limit:
@@ -699,12 +677,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_LIMIT,
-		 g_param_spec_uint (NM_DEVICE_VXLAN_LIMIT, "", "",
-		                    0, G_MAXUINT32, 0,
-		                    G_PARAM_READABLE |
-		                    G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_LIMIT] =
+	    g_param_spec_uint (NM_DEVICE_VXLAN_LIMIT, "", "",
+	                       0, G_MAXUINT32, 0,
+	                       G_PARAM_READABLE |
+	                       G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:dst-port:
@@ -714,12 +691,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_DST_PORT,
-		 g_param_spec_uint (NM_DEVICE_VXLAN_DST_PORT, "", "",
-		                    0, 65535, 0,
-		                    G_PARAM_READABLE |
-		                    G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_DST_PORT] =
+	    g_param_spec_uint (NM_DEVICE_VXLAN_DST_PORT, "", "",
+	                       0, 65535, 0,
+	                       G_PARAM_READABLE |
+	                       G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:src-port-min:
@@ -729,12 +705,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_SRC_PORT_MIN,
-		 g_param_spec_uint (NM_DEVICE_VXLAN_SRC_PORT_MIN, "", "",
-		                    0, 65535, 0,
-		                    G_PARAM_READABLE |
-		                    G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_SRC_PORT_MIN] =
+	    g_param_spec_uint (NM_DEVICE_VXLAN_SRC_PORT_MIN, "", "",
+	                       0, 65535, 0,
+	                       G_PARAM_READABLE |
+	                       G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:src-port-max:
@@ -744,12 +719,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_SRC_PORT_MAX,
-		 g_param_spec_uint (NM_DEVICE_VXLAN_SRC_PORT_MAX, "", "",
-		                    0, 65535, 0,
-		                    G_PARAM_READABLE |
-		                    G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_SRC_PORT_MAX] =
+	    g_param_spec_uint (NM_DEVICE_VXLAN_SRC_PORT_MAX, "", "",
+	                       0, 65535, 0,
+	                       G_PARAM_READABLE |
+	                       G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:proxy:
@@ -758,12 +732,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_PROXY,
-		 g_param_spec_boolean (NM_DEVICE_VXLAN_PROXY, "", "",
-		                       FALSE,
-		                       G_PARAM_READABLE |
-		                       G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_PROXY] =
+	    g_param_spec_boolean (NM_DEVICE_VXLAN_PROXY, "", "",
+	                          FALSE,
+	                          G_PARAM_READABLE |
+	                          G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:rsc:
@@ -772,12 +745,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_RSC,
-		 g_param_spec_boolean (NM_DEVICE_VXLAN_RSC, "", "",
-		                       FALSE,
-		                       G_PARAM_READABLE |
-		                       G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_RSC] =
+	    g_param_spec_boolean (NM_DEVICE_VXLAN_RSC, "", "",
+	                          FALSE,
+	                          G_PARAM_READABLE |
+	                          G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:l2miss:
@@ -786,12 +758,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_L2MISS,
-		 g_param_spec_boolean (NM_DEVICE_VXLAN_L2MISS, "", "",
-		                       FALSE,
-		                       G_PARAM_READABLE |
-		                       G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_L2MISS] =
+	    g_param_spec_boolean (NM_DEVICE_VXLAN_L2MISS, "", "",
+	                          FALSE,
+	                          G_PARAM_READABLE |
+	                          G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * NMDeviceVxlan:l3miss:
@@ -800,10 +771,11 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 	 *
 	 * Since: 1.2
 	 */
-	g_object_class_install_property
-		(object_class, PROP_L3MISS,
-		 g_param_spec_boolean (NM_DEVICE_VXLAN_L3MISS, "", "",
-		                       FALSE,
-		                       G_PARAM_READABLE |
-		                       G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_L3MISS] =
+	    g_param_spec_boolean (NM_DEVICE_VXLAN_L3MISS, "", "",
+	                          FALSE,
+	                          G_PARAM_READABLE |
+	                          G_PARAM_STATIC_STRINGS);
+
+	_nml_dbus_meta_class_init_with_properties (object_class, &_nml_dbus_meta_iface_nm_device_vxlan);
 }
