@@ -1,18 +1,5 @@
-/* This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA.
- *
+// SPDX-License-Identifier: LGPL-2.1+
+/*
  * Copyright (C) 2018 Red Hat, Inc.
  */
 
@@ -44,4 +31,7 @@ int nm_sd_dns_name_to_wire_format (const char *domain,
                                    size_t len,
                                    gboolean canonical);
 
+int nm_sd_dns_name_is_valid (const char *s);
+gboolean nm_sd_hostname_is_valid(const char *s, bool allow_trailing_dot);
+
 #endif /* __NM_SD_UTILS_SHARED_H__ */
