@@ -1,20 +1,5 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-/* nm-platform.c - Handle runtime kernel networking configuration
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
+// SPDX-License-Identifier: GPL-2.0+
+/*
  * Copyright (C) 2015 Red Hat, Inc.
  */
 
@@ -231,8 +216,10 @@ TEST_RECV_FRAME_DEFINE (_test_recv_data1_frame0,
 static void
 _test_recv_data1_check (GMainLoop *loop, NMLldpListener *listener)
 {
-	GVariant *neighbors, *attr;
+	GVariant *neighbors, *attr, *child;
 	gs_unref_variant GVariant *neighbor = NULL;
+	guint v_uint = 0;
+	const char *v_str = NULL;
 
 	neighbors = nm_lldp_listener_get_neighbors (listener);
 	nmtst_assert_variant_is_of_type (neighbors, G_VARIANT_TYPE ("aa{sv}"));
@@ -242,7 +229,7 @@ _test_recv_data1_check (GMainLoop *loop, NMLldpListener *listener)
 	                              SD_LLDP_CHASSIS_SUBTYPE_MAC_ADDRESS, "00:01:30:F9:AD:A0",
 	                              SD_LLDP_PORT_SUBTYPE_INTERFACE_NAME, "1/1");
 	g_assert (neighbor);
-	g_assert_cmpint (g_variant_n_children (neighbor), ==, 4 + 10);
+	g_assert_cmpint (g_variant_n_children (neighbor), ==, 4 + 16);
 
 	attr = g_variant_lookup_value (neighbor, NM_LLDP_ATTR_DESTINATION, G_VARIANT_TYPE_STRING);
 	nmtst_assert_variant_string (attr, NM_LLDP_DEST_NEAREST_BRIDGE);
@@ -270,11 +257,49 @@ _test_recv_data1_check (GMainLoop *loop, NMLldpListener *listener)
 	nmtst_assert_variant_uint32 (attr, 20);
 	nm_clear_g_variant (&attr);
 
-	/* unsupported: Management Address */
-	/* unsupported: IEEE 802.3 - Power Via MDI */
-	/* unsupported: IEEE 802.3 - MAC/PHY Configuration/Status */
+	/* Management Address */
+	attr = g_variant_lookup_value (neighbor, NM_LLDP_ATTR_MANAGEMENT_ADDRESSES, G_VARIANT_TYPE ("aa{sv}"));
+	g_assert (attr);
+	g_assert_cmpuint (g_variant_n_children (attr), ==, 1);
+	child = g_variant_get_child_value (attr, 0);
+	g_assert (child);
+	g_variant_lookup (child, "interface-number", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 1001);
+	g_variant_lookup (child, "interface-number-subtype", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 2);
+	g_variant_lookup (child, "address-subtype", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 6);
+	nm_clear_g_variant (&child);
+	nm_clear_g_variant (&attr);
+
+	/* IEEE 802.3 - Power Via MDI */
+	attr = g_variant_lookup_value (neighbor, NM_LLDP_ATTR_IEEE_802_3_POWER_VIA_MDI, G_VARIANT_TYPE_VARDICT);
+	g_assert (attr);
+	g_variant_lookup (attr, "mdi-power-support", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 7);
+	g_variant_lookup (attr, "pse-power-pair", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 1);
+	g_variant_lookup (attr, "power-class", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 0);
+	nm_clear_g_variant (&attr);
+
+	/* IEEE 802.3 - MAC/PHY Configuration/Status */
+	attr = g_variant_lookup_value (neighbor, NM_LLDP_ATTR_IEEE_802_3_MAC_PHY_CONF, G_VARIANT_TYPE_VARDICT);
+	g_assert (attr);
+	g_variant_lookup (attr, "autoneg", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 3);
+	g_variant_lookup (attr, "pmd-autoneg-cap", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 0x6c00);
+	g_variant_lookup (attr, "operational-mau-type", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 16);
+	nm_clear_g_variant (&attr);
+
 	/* unsupported: IEEE 802.3 - Link Aggregation */
-	/* unsupported: IEEE 802.3 - Maximum Frame Size*/
+
+	/* Maximum Frame Size */
+	attr = g_variant_lookup_value (neighbor, NM_LLDP_ATTR_IEEE_802_3_MAX_FRAME_SIZE, G_VARIANT_TYPE_UINT32);
+	nmtst_assert_variant_uint32 (attr, 1522);
+	nm_clear_g_variant (&attr);
 
 	/* IEEE 802.1 - Port VLAN ID */
 	attr = g_variant_lookup_value (neighbor, NM_LLDP_ATTR_IEEE_802_1_PVID, G_VARIANT_TYPE_UINT32);
@@ -289,6 +314,18 @@ _test_recv_data1_check (GMainLoop *loop, NMLldpListener *listener)
 	nmtst_assert_variant_uint32 (attr, 1);
 	nm_clear_g_variant (&attr);
 
+	/* new PPVID attributes */
+	attr = g_variant_lookup_value (neighbor, NM_LLDP_ATTR_IEEE_802_1_PPVIDS, G_VARIANT_TYPE ("aa{sv}"));
+	g_assert_cmpuint (g_variant_n_children (attr), ==, 1);
+	child = g_variant_get_child_value (attr, 0);
+	g_assert (child);
+	g_variant_lookup (child, "ppvid", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 0);
+	g_variant_lookup (child, "flags", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 1);
+	nm_clear_g_variant (&child);
+	nm_clear_g_variant (&attr);
+
 	/* IEEE 802.1 - VLAN Name */
 	attr = g_variant_lookup_value (neighbor, NM_LLDP_ATTR_IEEE_802_1_VLAN_NAME, G_VARIANT_TYPE_STRING);
 	nmtst_assert_variant_string (attr, "v2-0488-03-0505");
@@ -297,6 +334,18 @@ _test_recv_data1_check (GMainLoop *loop, NMLldpListener *listener)
 	nmtst_assert_variant_uint32 (attr, 488);
 	nm_clear_g_variant (&attr);
 
+	/* new VLAN attributes */
+	attr = g_variant_lookup_value (neighbor, NM_LLDP_ATTR_IEEE_802_1_VLANS, G_VARIANT_TYPE ("aa{sv}"));
+	g_assert_cmpuint (g_variant_n_children (attr), ==, 1);
+	child = g_variant_get_child_value (attr, 0);
+	g_assert (child);
+	g_variant_lookup (child, "vid", "u", &v_uint);
+	g_assert_cmpint (v_uint, ==, 488);
+	g_variant_lookup (child, "name", "&s", &v_str);
+	g_assert_cmpstr (v_str, ==, "v2-0488-03-0505");
+	nm_clear_g_variant (&child);
+	nm_clear_g_variant (&attr);
+
 	/* unsupported: IEEE 802.1 - Protocol Identity */
 }
 
@@ -393,7 +442,7 @@ _test_recv_fixture_setup (TestRecvFixture *fixture, gconstpointer user_data)
 
 	fixture->ifindex = link->ifindex;
 	fixture->fd = nm_steal_fd (&fd);
-	memcpy (fixture->mac, link->addr.data, ETH_ALEN);
+	memcpy (fixture->mac, link->l_address.data, ETH_ALEN);
 }
 
 typedef struct {
