@@ -1,18 +1,5 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-/* This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
+// SPDX-License-Identifier: GPL-2.0+
+/*
  * Copyright (C) 2010 Red Hat, Inc.
  */
 
@@ -29,6 +16,7 @@ char *nm_dhcp_dhclient_create_config (const char *interface,
                                       const char *hostname,
                                       guint32 timeout,
                                       gboolean use_fqdn,
+                                      NMDhcpHostnameFlags hostname_flags,
                                       const char *orig_path,
                                       const char *orig_contents,
                                       GBytes **out_new_client_id);
