@@ -1,20 +1,5 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-/* NetworkManager -- Network link manager
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
+// SPDX-License-Identifier: GPL-2.0+
+/*
  * Copyright (C) 2013 - 2014 Red Hat, Inc.
  */
 
@@ -30,7 +15,6 @@
 
 #include "nm-auth-subject.h"
 
-#include <string.h>
 #include <stdlib.h>
 
 #include "nm-dbus-manager.h"
@@ -174,9 +158,10 @@ _new_unix_process (GDBusMethodInvocation *context,
                    GDBusMessage *message)
 {
 	NMAuthSubject *self;
-	gboolean success = FALSE;
-	gulong pid = 0, uid = 0;
-	gs_free char *dbus_sender = NULL;
+	const char *dbus_sender = NULL;
+	gulong uid = 0;
+	gulong pid = 0;
+	gboolean success;
 
 	g_return_val_if_fail (context || (connection && message), NULL);
 
@@ -200,7 +185,7 @@ _new_unix_process (GDBusMethodInvocation *context,
 		return NULL;
 
 	g_return_val_if_fail (dbus_sender && *dbus_sender, NULL);
-	/* polkit glib library stores uid and pid as gint. There might be some
+	/* polkit glib library stores uid and pid as int. There might be some
 	 * pitfalls if the id ever happens to be larger then that. Just assert against
 	 * it here. */
 	g_return_val_if_fail (uid <= MIN (G_MAXINT, G_MAXINT32), NULL);
@@ -356,7 +341,7 @@ constructed (GObject *object)
 		return;
 	case NM_AUTH_SUBJECT_TYPE_UNIX_PROCESS:
 		/* Ensure pid and uid to be representable as int32.
-		 * DBUS treats them as uint32, polkit library as gint. */
+		 * DBUS treats them as uint32, polkit library as int. */
 		if (priv->unix_process.pid > MIN (G_MAXINT, G_MAXINT32))
 			break;
 		if (priv->unix_process.uid > MIN (G_MAXINT, G_MAXINT32)) {
