@@ -1,20 +1,5 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-/* NetworkManager -- Network link manager
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
+// SPDX-License-Identifier: GPL-2.0+
+/*
  * Copyright (C) 2006 - 2012 Red Hat, Inc.
  * Copyright (C) 2006 - 2008 Novell, Inc.
  */
@@ -37,7 +22,8 @@
 #include <systemd/sd-journal.h>
 #endif
 
-#include "nm-utils/nm-time-utils.h"
+#include "nm-glib-aux/nm-logging-base.h"
+#include "nm-glib-aux/nm-time-utils.h"
 #include "nm-errors.h"
 
 /*****************************************************************************/
@@ -68,6 +54,15 @@
 
 /*****************************************************************************/
 
+G_STATIC_ASSERT (LOG_EMERG   == 0);
+G_STATIC_ASSERT (LOG_ALERT   == 1);
+G_STATIC_ASSERT (LOG_CRIT    == 2);
+G_STATIC_ASSERT (LOG_ERR     == 3);
+G_STATIC_ASSERT (LOG_WARNING == 4);
+G_STATIC_ASSERT (LOG_NOTICE  == 5);
+G_STATIC_ASSERT (LOG_INFO    == 6);
+G_STATIC_ASSERT (LOG_DEBUG   == 7);
+
 /* We have more then 32 logging domains. Assert that it compiles to a 64 bit sized enum */
 G_STATIC_ASSERT (sizeof (NMLogDomain) >= sizeof (guint64));
 
@@ -90,22 +85,6 @@ typedef struct {
 	const char *name;
 } LogDesc;
 
-typedef struct {
-	const char *name;
-	const char *level_str;
-
-	/* nm-logging uses syslog internally. Note that the three most-verbose syslog levels
-	 * are LOG_DEBUG, LOG_INFO and LOG_NOTICE. Journal already highlights LOG_NOTICE
-	 * as special.
-	 *
-	 * On the other hand, we have three levels LOGL_TRACE, LOGL_DEBUG and LOGL_INFO,
-	 * which are regular messages not to be highlighted. For that reason, we must map
-	 * LOGL_TRACE and LOGL_DEBUG both to syslog level LOG_DEBUG. */
-	int syslog_level;
-
-	GLogLevelFlags g_log_level;
-} LogLevelDesc;
-
 typedef struct {
 	char *logging_domains_to_string;
 } GlobalMain;
@@ -164,16 +143,6 @@ NMLogDomain _nm_logging_enabled_state[_LOGL_N_REAL] = {
 
 /*****************************************************************************/
 
-static const LogLevelDesc level_desc[_LOGL_N] = {
-	[LOGL_TRACE] = { "TRACE", "<trace>", LOG_DEBUG,   G_LOG_LEVEL_DEBUG,   },
-	[LOGL_DEBUG] = { "DEBUG", "<debug>", LOG_DEBUG,   G_LOG_LEVEL_DEBUG,   },
-	[LOGL_INFO]  = { "INFO",  "<info>",  LOG_INFO,    G_LOG_LEVEL_INFO,    },
-	[LOGL_WARN]  = { "WARN",  "<warn>",  LOG_WARNING, G_LOG_LEVEL_MESSAGE, },
-	[LOGL_ERR]   = { "ERR",   "<error>", LOG_ERR,     G_LOG_LEVEL_MESSAGE, },
-	[_LOGL_OFF]  = { "OFF",   NULL,      0,           0,                   },
-	[_LOGL_KEEP] = { "KEEP",  NULL,      0,           0,                   },
-};
-
 static const LogDesc domain_desc[] = {
 	{ LOGD_PLATFORM,  "PLATFORM" },
 	{ LOGD_RFKILL,    "RFKILL" },
@@ -277,14 +246,8 @@ match_log_level (const char  *level,
                  NMLogLevel  *out_level,
                  GError     **error)
 {
-	int i;
-
-	for (i = 0; i < G_N_ELEMENTS (level_desc); i++) {
-		if (!g_ascii_strcasecmp (level_desc[i].name, level)) {
-			*out_level = i;
-			return TRUE;
-		}
-	}
+	if (_nm_log_parse_level (level, out_level))
+		return TRUE;
 
 	g_set_error (error, NM_MANAGER_ERROR, NM_MANAGER_ERROR_UNKNOWN_LOG_LEVEL,
 	             _("Unknown log level '%s'"), level);
@@ -302,7 +265,8 @@ nm_logging_setup (const char  *level,
 	NMLogDomain new_log_state[_LOGL_N_REAL];
 	NMLogLevel cur_log_level;
 	NMLogLevel new_log_level;
-	char **tmp, **iter;
+	gs_free const char **domains_v = NULL;
+	gsize i_d;
 	int i;
 	gboolean had_platform_debug;
 	gs_free char *domains_free = NULL;
@@ -337,28 +301,24 @@ nm_logging_setup (const char  *level,
 		}
 	}
 
-	tmp = g_strsplit_set (domains, ", ", 0);
-	for (iter = tmp; iter && *iter; iter++) {
+	domains_v = nm_utils_strsplit_set (domains, ", ");
+	for (i_d = 0; domains_v && domains_v[i_d]; i_d++) {
+		const char *s = domains_v[i_d];
+		const char *p;
 		const LogDesc *diter;
 		NMLogLevel domain_log_level;
 		NMLogDomain bits;
-		char *p;
 
 		/* LOGD_VPN_PLUGIN is protected, that is, when setting ALL or DEFAULT,
 		 * it does not enable the verbose levels DEBUG and TRACE, because that
 		 * may expose sensitive data. */
 		NMLogDomain protect = LOGD_NONE;
 
-		if (!strlen (*iter))
-			continue;
-
-		p = strchr (*iter, ':');
+		p = strchr (s, ':');
 		if (p) {
-			*p = '\0';
-			if (!match_log_level (p + 1, &domain_log_level, error)) {
-				g_strfreev (tmp);
+			*((char *) p) = '\0';
+			if (!match_log_level (p + 1, &domain_log_level, error))
 				return FALSE;
-			}
 		} else
 			domain_log_level = new_log_level;
 
@@ -372,26 +332,26 @@ nm_logging_setup (const char  *level,
 		}
 
 		/* Check for combined domains */
-		if (!g_ascii_strcasecmp (*iter, LOGD_ALL_STRING)) {
+		if (!g_ascii_strcasecmp (s, LOGD_ALL_STRING)) {
 			bits = LOGD_ALL;
 			protect = LOGD_VPN_PLUGIN;
-		} else if (!g_ascii_strcasecmp (*iter, LOGD_DEFAULT_STRING)) {
+		} else if (!g_ascii_strcasecmp (s, LOGD_DEFAULT_STRING)) {
 			bits = LOGD_DEFAULT;
 			protect = LOGD_VPN_PLUGIN;
-		} else if (!g_ascii_strcasecmp (*iter, LOGD_DHCP_STRING))
+		} else if (!g_ascii_strcasecmp (s, LOGD_DHCP_STRING))
 			bits = LOGD_DHCP;
-		else if (!g_ascii_strcasecmp (*iter, LOGD_IP_STRING))
+		else if (!g_ascii_strcasecmp (s, LOGD_IP_STRING))
 			bits = LOGD_IP;
 
 		/* Check for compatibility domains */
-		else if (!g_ascii_strcasecmp (*iter, "HW"))
+		else if (!g_ascii_strcasecmp (s, "HW"))
 			bits = LOGD_PLATFORM;
-		else if (!g_ascii_strcasecmp (*iter, "WIMAX"))
+		else if (!g_ascii_strcasecmp (s, "WIMAX"))
 			continue;
 
 		else {
 			for (diter = &domain_desc[0]; diter->name; diter++) {
-				if (!g_ascii_strcasecmp (diter->name, *iter)) {
+				if (!g_ascii_strcasecmp (diter->name, s)) {
 					bits = diter->num;
 					break;
 				}
@@ -400,7 +360,7 @@ nm_logging_setup (const char  *level,
 			if (!bits) {
 				if (!bad_domains) {
 					g_set_error (error, NM_MANAGER_ERROR, NM_MANAGER_ERROR_UNKNOWN_LOG_DOMAIN,
-					             _("Unknown log domain '%s'"), *iter);
+					             _("Unknown log domain '%s'"), s);
 					return FALSE;
 				}
 
@@ -408,7 +368,7 @@ nm_logging_setup (const char  *level,
 					g_string_append (unrecognized, ", ");
 				else
 					unrecognized = g_string_new (NULL);
-				g_string_append (unrecognized, *iter);
+				g_string_append (unrecognized, s);
 				continue;
 			}
 		}
@@ -429,7 +389,6 @@ nm_logging_setup (const char  *level,
 			}
 		}
 	}
-	g_strfreev (tmp);
 
 	g_clear_pointer (&gl_main.logging_domains_to_string, g_free);
 
@@ -544,27 +503,50 @@ _domains_to_string (gboolean include_level_override,
 	return g_string_free (str, FALSE);
 }
 
+static char _all_logging_domains_to_str[273];
+
 const char *
 nm_logging_all_domains_to_string (void)
 {
-	static GString *str;
+	static const char *volatile str = NULL;
+	const char *s;
 
-	if (G_UNLIKELY (!str)) {
+again:
+	s = g_atomic_pointer_get (&str);
+	if (G_UNLIKELY (!s)) {
+		static gsize once = 0;
 		const LogDesc *diter;
+		gsize buf_l;
+		char *buf_p;
 
-		str = g_string_new (LOGD_DEFAULT_STRING);
+		if (!g_once_init_enter (&once))
+			goto again;
+
+		buf_p = _all_logging_domains_to_str;
+		buf_l = sizeof (_all_logging_domains_to_str);
+
+		nm_utils_strbuf_append_str (&buf_p, &buf_l, LOGD_DEFAULT_STRING);
 		for (diter = &domain_desc[0]; diter->name; diter++) {
-			g_string_append_c (str, ',');
-			g_string_append (str, diter->name);
+			nm_utils_strbuf_append_c (&buf_p, &buf_l, ',');
+			nm_utils_strbuf_append_str (&buf_p, &buf_l, diter->name);
 			if (diter->num == LOGD_DHCP6)
-				g_string_append (str, "," LOGD_DHCP_STRING);
+				nm_utils_strbuf_append_str (&buf_p, &buf_l, ","LOGD_DHCP_STRING);
 			else if (diter->num == LOGD_IP6)
-				g_string_append (str, "," LOGD_IP_STRING);
+				nm_utils_strbuf_append_str (&buf_p, &buf_l, ","LOGD_IP_STRING);
 		}
-		g_string_append (str, "," LOGD_ALL_STRING);
+		nm_utils_strbuf_append_str (&buf_p, &buf_l, LOGD_ALL_STRING);
+
+		/* Did you modify the logging domains (or their names)? Adjust the size of
+		 * _all_logging_domains_to_str buffer above to have the exact size. */
+		nm_assert (strlen (_all_logging_domains_to_str) == sizeof (_all_logging_domains_to_str) - 1);
+		nm_assert (buf_l == 1);
+
+		s = _all_logging_domains_to_str;
+		g_atomic_pointer_set (&str, s);
+		g_once_init_leave (&once, 1);
 	}
 
-	return str->str;
+	return s;
 }
 
 /**
@@ -622,9 +604,11 @@ _iovec_set_string (struct iovec *iov, const char *str)
 	_iovec_set (iov, str, strlen (str));
 }
 
+#define _iovec_set_string_literal(iov, str) _iovec_set ((iov), ""str"", NM_STRLEN (str))
+
 _nm_printf (3, 4)
 static void
-_iovec_set_format (struct iovec *iov, gpointer *iov_free, const char *format, ...)
+_iovec_set_format (struct iovec *iov, char **iov_free, const char *format, ...)
 {
 	va_list ap;
 	char *str;
@@ -741,12 +725,15 @@ _nm_log_impl (const char *file,
 	case LOG_BACKEND_JOURNAL:
 		{
 			gint64 now, boottime;
-#define _NUM_MAX_FIELDS_SYSLOG_FACILITY 10
-			struct iovec iov_data[12 + _NUM_MAX_FIELDS_SYSLOG_FACILITY];
+			struct iovec iov_data[15];
 			struct iovec *iov = iov_data;
-			gpointer iov_free_data[5];
-			gpointer *iov_free = iov_free_data;
-			nm_auto_free_gstring GString *s_domain_all = NULL;
+			char *iov_free_data[5];
+			char **iov_free = iov_free_data;
+			const LogDesc *diter;
+			NMLogDomain dom_all;
+			char s_log_domains_buf[NM_STRLEN ("NM_LOG_DOMAINS=") + sizeof (_all_logging_domains_to_str)];
+			char *s_log_domains;
+			gsize l_log_domains;
 
 			now = nm_utils_get_monotonic_timestamp_ns ();
 			boottime = nm_utils_monotonic_timestamp_as_boottime (now, 1);
@@ -755,48 +742,25 @@ _nm_log_impl (const char *file,
 			_iovec_set_format (iov++, iov_free++, "MESSAGE="MESSAGE_FMT, MESSAGE_ARG (g->prefix, tv, msg));
 			_iovec_set_string (iov++, syslog_identifier_full (g->syslog_identifier));
 			_iovec_set_format_a (iov++, 30, "SYSLOG_PID=%ld", (long) getpid ());
-			{
-				const LogDesc *diter;
-				int i_domain = _NUM_MAX_FIELDS_SYSLOG_FACILITY;
-				const char *s_domain_1 = NULL;
-				NMLogDomain dom_all = domain;
-				NMLogDomain dom = dom_all & cur_log_state[level];
 
-				for (diter = &domain_desc[0]; diter->name; diter++) {
-					if (!NM_FLAGS_ANY (dom_all, diter->num))
-						continue;
+			dom_all = domain;
+			s_log_domains = s_log_domains_buf;
+			l_log_domains = sizeof (s_log_domains_buf);
 
-					/* construct a list of all domains (not only the enabled ones).
-					 * Note that in by far most cases, there is only one domain present.
-					 * Hence, save the construction of the GString. */
-					dom_all &= ~diter->num;
-					if (!s_domain_1)
-						s_domain_1 = diter->name;
-					else {
-						if (!s_domain_all) {
-							s_domain_all = g_string_new ("NM_LOG_DOMAINS=");
-							g_string_append (s_domain_all, s_domain_1);
-						}
-						g_string_append_c (s_domain_all, ',');
-						g_string_append (s_domain_all, diter->name);
-					}
-
-					if (NM_FLAGS_ANY (dom, diter->num)) {
-						if (i_domain > 0) {
-							/* SYSLOG_FACILITY is specified multiple times for each domain that is actually enabled. */
-							_iovec_set_format_str_a (iov++, 30, "SYSLOG_FACILITY=%s", diter->name);
-							i_domain--;
-						}
-						dom &= ~diter->num;
-					}
-					if (!dom && !dom_all)
-						break;
-				}
-				if (s_domain_all)
-					_iovec_set (iov++, s_domain_all->str, s_domain_all->len);
-				else
-					_iovec_set_format_str_a (iov++, 30, "NM_LOG_DOMAINS=%s", s_domain_1);
+			nm_utils_strbuf_append_str (&s_log_domains, &l_log_domains, "NM_LOG_DOMAINS=");
+			for (diter = &domain_desc[0]; dom_all != 0 && diter->name; diter++) {
+				if (!NM_FLAGS_ANY (dom_all, diter->num))
+					continue;
+				if (dom_all != domain)
+					nm_utils_strbuf_append_c (&s_log_domains, &l_log_domains, ',');
+				nm_utils_strbuf_append_str (&s_log_domains, &l_log_domains, diter->name);
+				dom_all &= ~diter->num;
 			}
+			nm_assert (l_log_domains > 0);
+			_iovec_set (iov++, s_log_domains_buf, s_log_domains - s_log_domains_buf);
+
+			G_STATIC_ASSERT_EXPR (LOG_FAC (LOG_DAEMON) == 3);
+			_iovec_set_string_literal (iov++, "SYSLOG_FACILITY=3");
 			_iovec_set_format_str_a (iov++, 15, "NM_LOG_LEVEL=%s", level_desc[level].name);
 			if (func)
 				_iovec_set_format (iov++, iov_free++, "CODE_FUNC=%s", func);
@@ -911,7 +875,7 @@ nm_log_handler (const char *log_domain,
 			                 "MESSAGE=%s%s", gl.imm.prefix, message ?: "",
 			                 syslog_identifier_full (gl.imm.syslog_identifier),
 			                 "SYSLOG_PID=%ld", (long) getpid (),
-			                 "SYSLOG_FACILITY=GLIB",
+			                 "SYSLOG_FACILITY=3",
 			                 "GLIB_DOMAIN=%s", log_domain ?: "",
 			                 "GLIB_LEVEL=%d", (int) (level & G_LOG_LEVEL_MASK),
 			                 "TIMESTAMP_MONOTONIC=%lld.%06lld", (long long) (now / NM_UTILS_NS_PER_SECOND), (long long) ((now % NM_UTILS_NS_PER_SECOND) / 1000),
