@@ -3,10 +3,11 @@
   Copyright © 2014 Axis Communications AB. All rights reserved.
 ***/
 
-#include "nm-sd-adapt.h"
+#include "nm-sd-adapt-core.h"
 
-#include <linux/filter.h>
 #include <arpa/inet.h>
+#include <linux/filter.h>
+#include <netinet/if_ether.h>
 
 #include "arp-util.h"
 #include "fd-util.h"
