@@ -229,9 +229,9 @@ if [[ -n "$BUILDDIR" ]]; then
 fi
 
 if ! _is_true "$NMTST_USE_VALGRIND" 0; then
-    "${NMTST_DBUS_RUN_SESSION[@]}" \
+    export NM_TEST_UNDER_VALGRIND=0
+    exec "${NMTST_DBUS_RUN_SESSION[@]}" \
     "$TEST" "$@"
-    exit $?
 fi
 
 if [[ -z "${NMTST_VALGRIND}" ]]; then
@@ -250,6 +250,7 @@ LOGFILE="${TEST}.valgrind-log"
 
 export G_SLICE=always-malloc
 export G_DEBUG=gc-friendly
+export NM_TEST_UNDER_VALGRIND=1
 "${NMTST_DBUS_RUN_SESSION[@]}" \
 "${NMTST_LIBTOOL[@]}" \
 "$NMTST_VALGRIND" \
@@ -294,12 +295,18 @@ fi
 if [ $HAS_ERRORS -eq 0 ]; then
     # valgrind doesn't support setns syscall and spams the logfile.
     # hack around it...
-    if [ "$TEST_NAME" = 'test-link-linux' -o \
-         "$TEST_NAME" = 'test-acd' ]; then
-        if [ -z "$(sed -e '/^--[0-9]\+-- WARNING: unhandled .* syscall: /,/^--[0-9]\+-- it at http.*\.$/d' "$LOGFILE")" ]; then
-            HAS_ERRORS=1
-        fi
-    fi
+    case "$TEST_NAME" in
+        'test-link-linux' | \
+        'test-acd' | \
+        'test-service-providers' | \
+        'test-remote-settings-client' | \
+        'test-secret-agent' | \
+        'test-nm-client' )
+            if [ -z "$(sed -e '/^--[0-9]\+-- WARNING: unhandled .* syscall: /,/^--[0-9]\+-- it at http.*\.$/d' "$LOGFILE")" ]; then
+                HAS_ERRORS=1
+            fi
+            ;;
+    esac
 fi
 
 if [ $HAS_ERRORS -eq 0 ]; then
